/**
 * @(#)InConsoleTraversal.java  6 November 2006
 *
 * @author Viera K. Proulx
 */
import edu.neu.ccs.*;
import edu.neu.ccs.console.*;
import edu.neu.ccs.util.*;
import edu.neu.ccs.gui.*;

import java.io.*;
import java.awt.*;
/** 
 * Functional (External) iterator for a list of cities 
 * to generate data from the user console input.
 */
class InConsoleTraversal implements Traversal, ConsoleAware {
  
  /*-------------------------------------------------------- 
   Member data
   *------------------------------------------------------*/
  /** A <CODE>{@link City City}</CODE> object buffer 
   * to hold data for one city at a time. 
   */
  protected City c;
  
  /** user indicated the end of input */
  protected boolean done = true;
  /** no data is available to generate at the next request. */
  protected boolean submitted = false;
  
  /*-------------------------------------------------------- 
   Constructor
   *------------------------------------------------------*/
  /**
   * The constructor.
   * <p>Requests the input to initialize the
   *  <CODE>{@link City City}</CODE> object buffer.</p>
   *  <p>Indicates the <em>end of input</em> if user aborts the request.</p>
   */
  InConsoleTraversal() { 
    done = false;
    read();          
  }
  
  /*--------------------------------------------------------
   Methods to implement the Traversal interface
   *------------------------------------------------------*/
  /**
   * <p>Request the input to initialize the
   *  <CODE>{@link City City}</CODE> object buffer with the next data.</p>
   *  <p>Indicate the <em>end of input</em> if user aborts the request.</p>
   * @see Traversal#getRest()
   */
  public Traversal getRest(){
    read();
    return this;
  }
  
  /**
   * <p>Produce the city buffer object.</p>
   * <p>Report an error if the end of file has been reached.</p>
   * @see Traversal#getFirst()
   */
  public Object getFirst() {
    return c;
  }
  
  /**
   * The end of input predicate.  
   *
   *@return true when the user has indicated the end of data.
   * @see Traversal#isEmpty()
   */
  public boolean isEmpty() {
    return done; 
  }
  
  /*--------------------------------------------------------------
   Methods
   *------------------------------------------------------------*/
  /**
   * <p>prompt the user for the data needed to initialize  
   * the city buffer.</p>
   * <p>On cancelled exception report the end of input
   * and record the new iterator state.</p>
   */

  public City read(){
    
    try{
      c = new City
      (console.in.requestInt("Zip code: "),
          console.in.requestString("City name: "),
          console.in.requestString("State: "),
          console.in.requestDouble("Longitude: "),
          console.in.requestDouble("Latitude: "));
    }
    catch(CancelledException e){
      console.out.println("End of input");
      this.done = true;
    }
    return c;
  }
  
}

