/**
 * @(#)InFileBufferedTraversal.java  6 November 2006
 *
 * @author Viera K. Proulx
 */
import edu.neu.ccs.*;
import edu.neu.ccs.gui.*;
import edu.neu.ccs.util.*;

import java.awt.*;
import java.io.*;
import javax.swing.*;
import java.util.*;
/** 
 * Functional (External) iterator for a list of cities 
 * to generate data from an input file.
 */
class InFileBufferedTraversal implements Traversal{
  
  /*-------------------------------------------------------- 
   Member data
   *------------------------------------------------------*/
  
  /** The input buffer for a buffered file reader. */
  protected BufferedReader buffer;
  
  /** A buffer that holds one line of input at a time. */
  protected String line;
  
  /** A <CODE>{@link City City}</CODE> object buffer 
   * to hold data for one city at a time. 
   */
  protected City c;
  
  /** Determines whether the general dialog has been closed. */
  protected boolean closed = true;
  
  /** Determines whether new baloon data has been submitted. */
  protected boolean submitted = false;
  
  /*-------------------------------------------------------- 
   Constructor
   *------------------------------------------------------*/ 
  /**
   * The constructor.
   * 
   * <p>Create a file chooser, ask the user to select a file to read,
   * initialize a <CODE>{@link BufferedReader BufferedReader}</CODE>
   * read the first line of data to initialize the 
   * <CODE>{@link City City}</CODE> buffer.</p>
   * <p>Display error messages when appropriate.</p>
   */
  public InFileBufferedTraversal() {
    
    /** build a file chooser 
     and have the user choose a file, 
     quitting this operation 
     if the user cancelled the choice */
    JFileChooser chooser = new JFileChooser(".");
    
    /** set the file extension to be .txt */
    chooser.setFileFilter(new FileView.ExtensionFileFilter("txt"));
    
    closed = false;
    
    /** see if file was selected - quit if user canceled */
    if (chooser.showOpenDialog(null) != JFileChooser.APPROVE_OPTION){
      closed = true;
      return;
    }
    try{
      buffer = new BufferedReader(
          new FileReader(chooser.getSelectedFile()));
    }
    catch(FileNotFoundException e){
      System.out.println("File not found exception: " + e);
      closed = true;        
    }
    
    /** read one line of input from the selected file */
    /** and extract values for the first city */
    getnext();
  }
  /*--------------------------------------------------------
   Methods to implement the Traversal interface
   *------------------------------------------------------*/
  
  /** 
   * Process the next item, if available.
   */
  public void getnext() {
    
    if (!closed)
      try{
        if (buffer.ready()){
          
          line = new String(buffer.readLine());
          if ((line == null)||(line == "")){     
            buffer.close();
            closed = true;
          }
          else{ 
            read(line);
          }            
        }
        else{
          buffer.close();
          closed = true;
        }
      }
    catch(Exception e) {
      System.out.println("Error in reading line exception: " + e);
      closed = true;
    }
    
  }
  
  /**
   * <p>Produce the traversal for the remainder of this file.</p>
   * <p>Report an error if the end of file has been reached.</p>
   * @see Traversal#getRest()
   */
  public Traversal getRest(){
    getnext();
    return this;
  }
  
  /**
   * <p>Produce the current element of this file.</p>
   * <p>Report an error if the end of file has been reached.</p>
   * @see Traversal#getFirst()
   */
  public Object getFirst() {
    
    if (!closed)
      return c;
    else{
      closed = true;
      return null;
    }
  }
  
  /**
   * The end of file predicate.  
   *
   *@return true when the end of file has been reached,
   * or when no file is available to read.
   * @see Traversal#isEmpty()
   */
  public boolean isEmpty() {
    return closed; 
  }
  
  /**
   * <p>Use the given <CODE>{@link String String}<CODE> to initialize 
   * the city buffer data.</p>
   * <p>On parsing error abort the 
   * input file iterator and report the problem.</p>
   * 
   * @param s the encoded source <CODE>{@link String String}<CODE> 
   * for the city data.
   */
  public void read(String s){
    
    try {
      c = new City();
      c.fromStringData(line);
    }
    catch(java.text.ParseException e){
      System.out.println(e.getMessage() + 
          " parse exception in line\n" + 
          line);
      c = null;
      closed = true;
    }   
  }  
}